#include "config.h"
#include <inttypes.h>
#include "str91x.h"

//-----------------------------------------------------------------------------

void
__flash_init(void)
{
    if (FMI.bbsr >= 3) {
	// booting from bank 0 (>= 256KB)

	volatile uint16_t *bank1 = (volatile uint16_t *)(FMI.nbbadr << 2);

	uint32_t myaddr = (uint32_t)__flash_init;
	if (myaddr < ((FMI.nbbadr << 2) - 1024) || myaddr >= 0x04000000) {
	    // running either in bank 1 or in SRAM
	    uint16_t cr = 0;
	    if (__Config_FMICLK <= 66000000)
		cr |= Flash_CR_1WS;
	    else if (__Config_FMICLK <= 75000000)
		cr |= Flash_CR_2WS | Flash_CR_BUSCFG;
	    else 
		cr |= Flash_CR_3WS | Flash_CR_BUSCFG;
	    bank1[0] = Flash_Cmd_CR;
	    bank1[cr >> 1] = 0x03;

	    bank1[0] = Flash_Cmd_RD;
	}
    }
}

//-----------------------------------------------------------------------------

static void
clock(uint32_t clk)
{
    SCU.clkcntr = clk;
    asm volatile ("nop");
    asm volatile ("nop");
    asm volatile ("nop");
    asm volatile ("nop");
    asm volatile ("nop");
    asm volatile ("nop");
    asm volatile ("nop");
    asm volatile ("nop");
    asm volatile ("nop");
    asm volatile ("nop");
}

void
__scu_init(void)
{
    uint32_t clk = 0;

#if __Config_MCLK > 96000000
    #error MCLK > 96000000
#endif

#if __Config_RCLK == __Config_MCLK/1
    clk |= SCU_CLK_RCLK_DIV1;
#elif __Config_RCLK == __Config_MCLK/2
    clk |= SCU_CLK_RCLK_DIV2;
#elif __Config_RCLK == __Config_MCLK/4
    clk |= SCU_CLK_RCLK_DIV4;
#elif __Config_RCLK == __Config_MCLK/8
    clk |= SCU_CLK_RCLK_DIV8;
#elif __Config_RCLK == __Config_MCLK/16
    clk |= SCU_CLK_RCLK_DIV16;
#elif __Config_RCLK == __Config_MCLK/1024
    clk |= SCU_CLK_RCLK_DIV1024;
#else
    #error Invalid RCLK
#endif

#if __Config_HCLK == __Config_RCLK/1
    clk |= SCU_CLK_HCLK_DIV1;
#elif __Config_HCLK == __Config_RCLK/2
    clk |= SCU_CLK_HCLK_DIV2;
#elif __Config_HCLK == __Config_RCLK/4
    clk |= SCU_CLK_HCLK_DIV4;
#else
    #error Invalid HCLK
#endif

#if __Config_PCLK > __Config_HCLK
    #error PCLK > HCLK
#elif __Config_PCLK > 48000000
    #error PCLK > 48000000
#elif __Config_PCLK == __Config_RCLK/1
    clk |= SCU_CLK_PCLK_DIV1;
#elif __Config_PCLK == __Config_RCLK/2
    clk |= SCU_CLK_PCLK_DIV2;
#elif __Config_PCLK == __Config_RCLK/4
    clk |= SCU_CLK_PCLK_DIV4;
#elif __Config_PCLK == __Config_RCLK/8
    clk |= SCU_CLK_PCLK_DIV8;
#else
    #error Invalid PCLK
#endif

#if __Config_BRCLK == __Config_MCLK/1
    clk |= SCU_CLK_BR_FULL;
#elif __Config_BRCLK == __Config_MCLK/2
    clk |= SCU_CLK_BR_HALF;
#else
    #error Invalid BRCLK
#endif

#if __Config_MCLK == 48000000
    clk |= SCU_CLK_USB_FULL;
#elif __Config_MCLK == 96000000
    clk |= SCU_CLK_USB_HALF;
#else
    clk |= SCU_CLK_USB_EXT;
#endif

#if __Config_FMICLK == __Config_RCLK/1
    clk |= SCU_CLK_FMI_FULL;
#elif __Config_FMICLK == __Config_RCLK/2
    clk |= SCU_CLK_FMI_HALF;
#else
    #error Invalid FMICLK
#endif

#if __Config_EMICLK > 66000000
    #error EMICLK > 66000000
#elif __Config_EMICLK == __Config_HCLK/1
    clk |= SCU_CLK_EMI_FULL;
#elif __Config_EMICLK == __Config_HCLK/2
    clk |= SCU_CLK_EMI_HALF;
#else
    #error Invalid EMICLK
#endif

    clock(clk | SCU_CLK_MCLK_OSC);			// switch to OSC, define peripheral clocks

    SCU.pllconf &= ~SCU_PLL_EN;				// disable PLL

#if __Config_MCLK != F_CRYSTAL
    // ----------------------------------------
    // configure and switch to PLL clock

    SCU.sysstatus |= SCU_SYS_LOCK;			// reset lock bit

  #if F_CRYSTAL == 25000000 && __Config_MCLK == 96000000
    SCU.pllconf = SCU_PLL_96MHz;			// set PLL parameters
  #elif F_CRYSTAL == 25000000 && __Config_MCLK == 48000000
    SCU.pllconf = SCU_PLL_48MHz;
  #else
    #error PLL
  #endif
    SCU.pllconf |= SCU_PLL_EN;				// enable PLL

    while (!(SCU.sysstatus & SCU_SYS_LOCK))		// wait for lock
	;

    clock(clk | SCU_CLK_MCLK_PLL);			// switch to PLL
#endif
   
}
