#ifndef _uart_hh_
#define _uart_hh_

//-----------------------------------------------------------------------------

#ifndef BufferedUART
  #define BufferedUART	1	// interrupt controlled?
#endif

//
// Requires:
//	BRCLK		Baud rate clock frequency.
//

//-----------------------------------------------------------------------------

class UARTbase {
public:
    enum Parity { None, Even, Odd, Mark, Space };
    enum Error { Empty=-1, Overrun=-2, Parity=-3, Framing=-4, Break=-5 };

    void rate (unsigned rate);
    void parity (int parity);

    void clear ();
    void set_break ();
    void clear_break ();

    int  get (bool wait);
    void put (char c);
    void puts (const char *);
    void putu (unsigned value, int ndigits=0, int radix=10);
    void puti (int value, int ndigits=0);
    void flush ();

    void interrupt ();				// interrupt handler

protected:
    UARTbase (int no, uint8_t *rxp, int rxsz, uint8_t *txp, int txsz)
	{ init(no, rxp, rxsz, txp, txsz); }
    void	init (int no, uint8_t *rxp, int rxsz, uint8_t *txp, int txsz);
    void	start ();

    unsigned	disable (void);			// returns prior state
    void	restore (unsigned state);	// conditionally re-enable

private:
    volatile UART_t	*uart;
};

//-----------------------------------------------------------------------------

template<int no, int rxsz, int txsz>
class UARTc : public UARTbase {
public:
    UARTc () : UARTbase(no,
			rxb, (rxsz > 16) ? (rxsz - 16) : 0,
			txb, (txsz > 16) ? (txsz - 16) : 0) {}
    void start ();
protected:
    uint8_t rxb[(rxsz > 16) ? (rxsz - 16) : 1];
    uint8_t txb[(txsz > 16) ? (txsz - 16) : 1];
};

//-----------------------------------------------------------------------------

template<int no, int rxsz, int txsz>
void
UARTc<no,rxsz,txsz>::start()
{
  UARTbase::start();
}

//-----------------------------------------------------------------------------
#endif // _uart_hh_
