#ifndef __ARM966_CORE_H__
#define __ARM966_CORE_H__

//-----------------------------------------------------------------------------
#ifdef __thumb__

extern uint32_t	arm9_get_idcode(void);
extern uint32_t arm9_get_tcmsize(void);
extern uint32_t arm9_get_control(void);
extern void	arm9_set_control(uint32_t value);
extern void	arm9_wait_for_irq(void);
extern void	arm9_drain_buffers(void);
extern uint32_t arm9_get_config(void);
extern void	arm9_set_config(uint32_t value);

//-----------------------------------------------------------------------------
#else

static inline uint32_t arm9_get_idcode(void)
{ 
    uint32_t r;
    asm volatile ("MRC	p15, 0, %0, c0, c0, 0" : "=r"(r));
    return r;
}
static inline uint32_t arm9_get_tcmsize(void)
{
    uint32_t r;
    asm volatile ("MRC	p15, 0, %0, c0, c0, 2" : "=r"(r));
    return r;
}
static inline uint32_t arm9_get_control(void)
{
    uint32_t r;
    asm volatile ("MRC	p15, 0, %0, c1, c0, 0" : "=r"(r));
    return r;
}
static inline void	arm9_set_control(uint32_t value)
{
    asm volatile ("MCR	p15, 0, %0, c1, c0, 0" :: "r"(value));
}
static inline void	arm9_wait_for_irq(void)
{
    asm volatile ("MRC	p15, 0, r0, c7, c0, 4");
}
static inline void	arm9_drain_buffers(void)
{
    asm volatile ("MCR	p15, 0, r0, c7, c10, 4");
}
static inline uint32_t arm9_get_config(void)
{
    uint32_t r;
    asm volatile ("MRC	p15, 0, %0, c1, c0, 0" : "=r"(r));
    return r;
}
static inline void	arm9_set_config(uint32_t value)
{
    asm volatile ("MCR	p15, 0, %0, c1, c0, 0" :: "r"(value));
}

#endif
//-----------------------------------------------------------------------------

static inline void
arm9_control(uint32_t mask, uint32_t value)
{
    uint32_t r = arm9_get_control();
    arm9_set_control((r & ~mask) | (value & mask));
}

#define ARM9_CONTROL_LDPC_T_MASK	(0<<15)
#define ARM9_CONTROL_LDPC_T_SET		(0<<15)
#define ARM9_CONTROL_LDPC_T_IGNORE	(1<<15)
#define ARM9_CONTROL_ITCM_MASK		(1<<12)
#define ARM9_CONTROL_ITCM_DISABLE	(0<<12)
#define ARM9_CONTROL_ITCM_ENABLE	(1<<12)
#define ARM9_CONTROL_AHB_BUFFER_MASK	(1<<3)
#define ARM9_CONTROL_AHB_BUFFER_DISABLE	(0<<3)
#define ARM9_CONTROL_AHB_BUFFER_ENABLE	(1<<3)
#define ARM9_CONTROL_DTCM_MASK		(1<<2)
#define ARM9_CONTROL_DTCM_DISABLE	(0<<2)
#define ARM9_CONTROL_DTCM_ENABLE	(1<<2)
#define ARM9_CONTROL_ALIGNMENT_MASK	(1<<1)
#define ARM9_CONTROL_ALIGNMENT_IGNORE	(0<<1)
#define ARM9_CONTROL_ALIGNMENT_FAULT	(1<<1)
// more bits in docs

static inline void
arm9_config(uint32_t mask, uint32_t value)
{
    uint32_t r = arm9_get_config();
    arm9_set_config((r & ~mask) | (value & mask));
}

#define ARM9_CONFIG_ITCM_MASK		(1<<18)
#define ARM9_CONFIG_ITCM_UNORDERED	(0<<18)
#define ARM9_CONFIG_ITCM_ORDERED	(1<<18)
#define ARM9_CONFIG_DTCM_MASK		(1<<17)
#define ARM9_CONFIG_DTCM_UNORDERED	(0<<17)
#define ARM9_CONFIG_DTCM_ORDERED	(1<<17)
#define ARM9_CONFIG_AHB_PREFETCH_MASK	 (1<<16)
#define ARM9_CONFIG_AHB_PREFETCH_DISABLE (0<<16)
#define ARM9_CONFIG_AHB_PREFETCH_ENABLE	 (1<<16)
// more bits in docs

//-----------------------------------------------------------------------------
#endif //__ARM966_CORE_H__
