#include "config.h"
#include <inttypes.h>
#include "str91x.h"
#include "UART.hh"
#include "rprintf.h"

UARTc<1,0,0> uart;

extern "C" {
    void rprintc(unsigned c) { uart.put(c); }
    void rflush()            { uart.flush(); }
}

//-----------------------------------------------------------------------------

unsigned data[1000] = {0};

/*
 * 25MHz basic				14
 * 48MHz no PFQ/BC	unordered	11
 * 48MHz PFQ/BC		unordered	 5
 * 96MHz no PFQ/BC	ordered		20	(FMI=MCLK/2)
 * 96MHz PFQ/BC		ordered		 5	(FMI=MCLK/2)
 */
unsigned
test1()
{
    unsigned t = TIM0.cntr;

    unsigned count = 1000;
#ifdef __thumb__
    asm volatile ("1: sub %0,#1\n\tbne 1b" : "+r"(count));
#else
    asm volatile ("1: subs %0,%0,#1\n\tbne 1b" : "+r"(count));
#endif

    return TIM0.cntr - t;
}

/*			Thumb	ARM
 * 25MHz PFQ/BC		21
 * 48MHz PFQ/BC		21	18
 * 96MHz PFQ/BC		28	32
 */
unsigned
test2()
{
    unsigned t = TIM0.cntr;

    unsigned count = 1000;
    asm volatile ("1:\n\t"
		  "	nop		\n\t"
		  "	nop		\n\t"
		  "	nop		\n\t"
		  "	nop		\n\t"
		  "	nop		\n\t"
		  "	nop		\n\t"
		  "	nop		\n\t"
		  "	nop		\n\t"
		  "	nop		\n\t"
		  "	nop		\n\t"
#ifdef __thumb__
		  "	sub %0,#1	\n\t"
#else
		  "	subs %0,%0,#1	\n\t"
#endif
		  "	bne 1b" : "+r"(count));

    return TIM0.cntr - t;
}

/*			Thumb	ARM
 * 25MHz PFQ/BC		
 * 48MHz PFQ/BC			28
 * 96MHz PFQ/BC			52
 */
unsigned
test3()
{
    unsigned t = TIM0.cntr;

    unsigned count = 1000;
    asm volatile ("1:\n\t"
		  "	nop		\n\t"
		  "	nop		\n\t"
		  "	nop		\n\t"
		  "	nop		\n\t"
		  "	nop		\n\t"
		  "	nop		\n\t"
		  "	nop		\n\t"
		  "	nop		\n\t"
		  "	nop		\n\t"
		  "	nop		\n\t"
		  "	nop		\n\t"
		  "	nop		\n\t"
		  "	nop		\n\t"
		  "	nop		\n\t"
		  "	nop		\n\t"
		  "	nop		\n\t"
		  "	nop		\n\t"
		  "	nop		\n\t"
		  "	nop		\n\t"
		  "	nop		\n\t"
#ifdef __thumb__
		  "	sub %0,#1	\n\t"
#else
		  "	subs %0,%0,#1	\n\t"
#endif
		  "	bne 1b" : "+r"(count));

    return TIM0.cntr - t;
}

/*				ARM
 * <data> in ROM	15 @48MHz	22 @96MHz
 * <data> in RAM	 9 @48MHz	14 @96MHz
 */
unsigned
test4()
{
    unsigned t = TIM0.cntr, x;
    const unsigned *p = data;

    unsigned count = 1000;
    asm volatile ("1:\n\t"
		  "	ldr %1,[%2],#4	\n\t"
#ifdef __thumb__
		  "	sub %0,#1	\n\t"
#else
		  "	subs %0,%0,#1	\n\t"
#endif
		  "	bne 1b" : "+r"(count), "=r"(x), "+r"(p));

    return TIM0.cntr - t;
}

/*
 *  1x LDR		  9
 *  4x LDR		 15	=> 2/LDR fr 1..4	ARM9:1+interlock
 *  5x LDR		 53
 *  6x LDR		 63
 * 10x LDR		103	=> 10/LDR ab 5
 *
 * 10x LDR, 1WS		114
 *
 *  4x LDR, 6x NOP	 24
 *  5x LDR, 5x NOP	 58	Anordnung ist egal.
 */
unsigned
test5(unsigned count)
{
    unsigned t = TIM0.cntr, temp = 0;
    const unsigned *p = data;

    asm volatile ("1:\n\t"
		  "	ldr %2,[%1,%2]	\n\t"
		  "	ldr %2,[%1,%2]	\n\t"
		  "	ldr %2,[%1,%2]	\n\t"
		  "	ldr %2,[%1,%2]	\n\t"
		  "	ldr %2,[%1,%2]	\n\t"
		  "	ldr %2,[%1,%2]	\n\t"
		  "	ldr %2,[%1,%2]	\n\t"
		  "	ldr %2,[%1,%2]	\n\t"
		  "	ldr %2,[%1,%2]	\n\t"
		  "	ldr %2,[%1,%2]	\n\t"
		  "	subs %0,%0,#1	\n\t"
		  "	bne 1b" : "+r"(count), "+r"(p), "+r"(temp));

    return TIM0.cntr - t;
}

/*
 * RAM/DTCM:
 * 1x  LDM	11/iter
 * 10x LDM	65/iter
 *	=> LDM:6. ARM9:5.
 *
 * RAM/AHB/buffered/unbuffered:
 * 10x LDM	135/iter
 *	=> LDM:13.
 *
 * ROM:
 * 1x LDM	 27  /item
 * 5x LDM	 95  /item
 * 10x LDM	176.5/item
 *	=> LDM:16.5-17
 */
unsigned
test6(unsigned count)
{
    unsigned t = TIM0.cntr, temp1, temp2, temp3, temp4, temp5;
//    const unsigned *p = data;
//    const unsigned *p = (const unsigned *)0x40000000;
//    const unsigned *p = (const unsigned *)0x50000000;
    const unsigned *p = (const unsigned *)0x10000;

    asm volatile ("1:\n\t"
/*		  "	ldmia %1,{%2,%3,%4,%5,%6}	\n\t"
		  "	ldmia %1,{%2,%3,%4,%5,%6}	\n\t"
		  "	ldmia %1,{%2,%3,%4,%5,%6}	\n\t"
		  "	ldmia %1,{%2,%3,%4,%5,%6}	\n\t"
		  "	ldmia %1,{%2,%3,%4,%5,%6}	\n\t"
		  "	ldmia %1,{%2,%3,%4,%5,%6}	\n\t"
		  "	ldmia %1,{%2,%3,%4,%5,%6}	\n\t"
		  "	ldmia %1,{%2,%3,%4,%5,%6}	\n\t"
		  "	ldmia %1,{%2,%3,%4,%5,%6}	\n\t"	*/
		  "	ldmia %1,{%2,%3,%4,%5,%6}	\n\t"
		  "	subs %0,%0,#1			\n\t"
		  "	bne 1b" : "+r"(count), "+r"(p),
		  "+r"(temp1),"+r"(temp2),"+r"(temp3),"+r"(temp4),"+r"(temp5));

    return TIM0.cntr - t;
}

//-----------------------------------------------------------------------------

const uint32_t scu_per1 = SCU_PR1_GPIO(0) | SCU_PR1_GPIO(1) | SCU_PR1_GPIO(2) | SCU_PR1_GPIO(3)
    | SCU_PR1_GPIO(4) | SCU_PR1_GPIO(5) | SCU_PR1_GPIO(7) | SCU_PR1_GPIO(9)
    | SCU_PR1_UART(1) | SCU_PR1_TIM01;

void __sys_init()
{
    __flash_init();
    __scu_init();

    // enable peripheral clocks
    SCU.pcgr1 |= scu_per1 & SCU_PCGR1_MASK;

    // peripheral reset
    SCU.prr1 &= ~(scu_per1 & SCU_PRR1_MASK);	// activate reset
    SCU.prr1 |= (scu_per1 & SCU_PRR1_MASK);	// release reset

    // SCU.mgr default (all clocks stopped in idle mode)
    // SCU.pecgr default (all clocks enabled in debug mode)

    SCU.emi          = SCU_EMI_GPIO;		// port8/9 is GPIO
    GPIODIR(9)	     = 1<<0;			// LED
    GPIODATA(9,1<<0) = 1<<0;			// LED on

    SCU.gpioin[3]  = SCU_GIN_ALT1(2);		// UART1 Rx
    SCU.gpioout[3] = SCU_GOUT_ALT2(3);		// UART1 Tx

    SCU.gpioin[5]  = SCU_GIN_ALT1(0);		// CAN Rx
    
    SCU.gpioout[5] = SCU_GOUT_ALT2(1);		// CAN Tx

}

//-----------------------------------------------------------------------------

void
initialize(void)
{
    uart.rate(115200);
    uart.start();
}

int
main(int argc, char **argv)
{
    initialize();

    uart.put('\n');
    uart.puts(__DATE__ "\n");
    uart.puts(__TIME__ "\n");

    SCU.scr0 |= SCU_SCR0_PFQBC_EN;
    SCU.scr0 &= ~SCU_SCR0_DTCM_1WS;

    TIM0.cntr = 0;
    TIM0.cr1 = TIM_CR1_EN;

    for (;;) {
	unsigned t1 = test5(100) & 0xFFFF;
	rprintf("%u %u - ", t1, t1 * (__Config_MCLK/__Config_PCLK));
	unsigned t2 = test5(200) & 0xFFFF;
	rprintf("%u %u: ", t2, t2 * (__Config_MCLK/__Config_PCLK));
	unsigned t = t2 - t1;
	rprintf("%u\n", t * (__Config_MCLK/__Config_PCLK));
 
	for (volatile unsigned i = 0; i < 1000000; ++i)
	    ;
    }

    return 0;
}
