#ifndef __PL011_UART_H__
#define __PL011_UART_H__

#include <inttypes.h>

typedef struct UART_t {
    uint16_t	dr;		//+00	data register
    uint16_t		_02;
    uint16_t	rsr;		//+04	receive status
    uint16_t		_04[9];
    uint16_t	fr;		//+18	flag register
    uint16_t		_1A[3];
    uint16_t	ilpr;		//+20	IrDA low-power counter
    uint16_t		_22;
    uint16_t	ibrd;		//+24	integer baud rate
    uint16_t		_26;
    uint16_t	fbrd;		//+28	fractional baud rate
    uint16_t		_2A;
    uint16_t	lcr;		//+2C	line control
    uint16_t		_2E;
    uint16_t	cr;		//+30	control
    uint16_t		_32;
    uint16_t	ifls;		//+34	interrupt FIFO level
    uint16_t		_36;
    uint16_t	imsc;		//+38	interrupt mask set/clear
    uint16_t		_3A;
    uint16_t	ris;		//+3C	raw interrupt status
    uint16_t		_3E;
    uint16_t	mis;		//+40	masked interrupt status
    uint16_t		_42;
    uint16_t	icr;		//+44	interrupt clear
    uint16_t		_46;
    uint16_t	dmacr;		//+48	DMA control
    uint16_t		_4A;
} UART_t;

#define UART_DR_OE	(1<<11)		// overrun error
#define UART_DR_BE	(1<<10)		// break error
#define UART_DR_PE	(1<<9)		// parity error
#define UART_DR_FE	(1<<8)		// framing error
#define UART_DR_DATA	0xFF

#define UART_RSR_OE	(1<<3)		// overrun error	- reset on any write
#define UART_RSR_BE	(1<<2)		// break error		- reset on any write
#define UART_RSR_PE	(1<<1)		// parity error		- reset on any write
#define UART_RSR_FE	(1<<0)		// framing error	- reset on any write

#define UART_FR_RI	(1<<8)		// ring indicator
#define UART_FR_TXFE	(1<<7)		// Tx FIFO empty (Tx may still be busy)
#define UART_FR_RXFF	(1<<6)		// Rx FIFO full
#define UART_FR_TXFF	(1<<5)		// Tx FIFO full
#define UART_FR_RXFE	(1<<4)		// Rx FIFO empty
#define UART_FR_BUSY	(1<<3)		// Tx busy transmitting data
#define UART_FR_DCD	(1<<2)		// DCD active
#define UART_FR_DSR	(1<<1)		// DST active
#define UART_FR_CTS	(1<<0)		// CTS active

#define UART_LCR_SPS	(1<<7)		// stick parity
#define UART_LCR_WLEN8	(3<<5)		// word length
#define UART_LCR_WLEN7	(2<<5)		//
#define UART_LCR_WLEN6	(1<<5)		//
#define UART_LCR_WLEN5	(0<<5)		//
#define UART_LCR_FEN	(1<<4)		// enable FIFOs
#define UART_LCR_STP1	(0<<3)		// 1 stop bit
#define UART_LCR_STP2	(1<<3)		// 2 stop bits
#define UART_LCR_ODD	(0<<2)		// odd  parity
#define UART_LCR_EVEN	(1<<2)		// even parity
#define UART_LCR_PEN	(1<<1)		// parity enable
#define UART_LCR_BRK	(1<<0)		// send break

#define UART_CR_CTSEn	(1<<15)		// CTS hardware flow control
#define UART_CR_RTSEn	(1<<14)		// RTS hardware flow control
#define UART_CR_Out2	(1<<13)		// RI  output			!STR9x
#define UART_CR_Out1	(1<<12)		// DCD output			!STR9x
#define UART_CR_RTS	(1<<11)		// RTS output
#define UART_CR_DTR	(1<<10)		// DTR output
#define UART_CR_RXE	(1<<9)		// receive enable
#define UART_CR_TXE	(1<<8)		// transmit enable
#define UART_CR_LBE	(1<<7)		// loopback enable
#define UART_CR_SIRLP	(1<<2)		// IrDA SIR low power mode
#define UART_CR_SIREN	(1<<1)		// IrDA SIR enable
#define UART_CR_ENABLE	(1<<0)		// UART enable

#define UART_IFLS_RX2	(0<<3)		// Rx FIFO interrupt when >= 1/8 full
#define UART_IFLS_RX4	(1<<3)		// Rx FIFO interrupt when >= 2/8 full
#define UART_IFLS_RX8	(2<<3)		// Rx FIFO interrupt when >= 4/8 full
#define UART_IFLS_RX12	(3<<3)		// Rx FIFO interrupt when >= 6/8 full
#define UART_IFLS_RX14	(4<<3)		// Rx FIFO interrupt when >= 7/8 full
#define UART_IFLS_TX2	(0<<0)		// Tx FIFO interrupt when <= 1/8 full
#define UART_IFLS_TX4	(1<<0)		// Tx FIFO interrupt when <= 2/8 full
#define UART_IFLS_TX8	(2<<0)		// Tx FIFO interrupt when <= 4/8 full
#define UART_IFLS_TX12	(3<<0)		// Tx FIFO interrupt when <= 6/8 full
#define UART_IFLS_TX14	(4<<0)		// Tx FIFO interrupt when <= 8/8 full

#define UART_INT_OE	(1<<10)		// overrun error
#define UART_INT_BE	(1<<9)		// break error
#define UART_INT_PE	(1<<8)		// parity error
#define UART_INT_FE	(1<<7)		// framing error
#define UART_INT_RT	(1<<6)		// receive timeout
#define UART_INT_TX	(1<<5)		// transmit
#define UART_INT_RX	(1<<4)		// receive
#define UART_INT_DSR	(1<<3)		// DSR line
#define UART_INT_DCD	(1<<2)		// DCD line
#define UART_INT_CTS	(1<<1)		// CTS line
#define UART_INT_RI	(1<<0)		// RI line

#define UART_DMACR_ERR	(1<<2)		// DMA on error
#define UART_DMACR_TX	(1<<1)		// enable transmit DMA
#define UART_DMACR_RX	(1<<0)		// enable receive DMA

#endif //__PL011_UART_H__
