#ifndef __STR91x_FMI_H__
#define __STR91x_FMI_H__

typedef struct FMI_t {
    uint32_t	bbsr;		//+00 boot bank size register (32K*(2^n))	(power-on:0, reset:unchanged)
    uint32_t	nbbsr;		//+04 non-boot bank size register (8K*(2^n))	(power-on:0, reset:unchanged)
    uint32_t	_08;
    uint32_t	bbadr;		//+0C boot bank address register		(power-on:0, reset:unchanged)
    uint32_t	nbbadr;		//+10 non-boot bank address register		(power-on:0, reset:unchanged)
    uint32_t	_14;
    uint32_t	cr;		//+18 control register				(*is* affected by any reset)
    uint32_t	sr;		//+1C status register
    uint32_t	bce5addr;	//+20 5th entry target address register
} FMI_t;

#define FMI_CR_1WWS		(0<<8)		// 1 wait state for write access
#define FMI_CR_2WWS		(1<<8)		// 2 wait states for write access
#define FMI_CR_OMIE		(1<<7)		// out of memory interrupt enable
#define FMI_CR_BERRIE		(1<<6)		// flash bank error interrupt enable
#if 0
   // According to the documentation:
   #define FMI_CR_B1EN		(1<<4)		// flash bank 1 enable
   #define FMI_CR_B0EN		(1<<3)		// flash bank 0 enable
#else
   // As discovered by ST (http://www.stmcu.com/forums-cat-5212-21.html, 12.4.2007):
   #define FMI_CR_NNBEN		(1<<4)		// non-boot flash bank enable	(reset:0)
   #define FMI_CR_BBEN		(1<<3)		// boot     flash bank enable	(reset:1)
#endif 

#define FMI_SR_PFQBCENC		(1<<4)		// PFQ/BC enabled
#define FMI_SR_OM		(1<<3)		// out of memory error occurred
#define FMI_SR_B1ERR		(1<<1)		// a flash bank 1 error occurred
#define FMI_SR_B0ERR		(1<<1)		// a flash bank 0 error occurred

// subset
#define Flash_Cmd_RD		0xFF		// read array (normal state)
#define Flash_Cmd_RSIG		0x90		// read electronic signature
#define Flash_Cmd_ROTP		0x98		// read OTP sector
#define Flash_Cmd_CLSR		0x50		// clear status register
#define Flash_Cmd_CR		0x60		// write flash configuration register (next byte)

#define Flash_CR_1WS		(0<<11)		// 1 wait state (<= 66MHz)
#define Flash_CR_2WS		(1<<11)		// 2 wait states (<= 75MHz)
#define Flash_CR_3WS		(2<<11)		// 3 wait states (>  75MHz)
#define Flash_CR_PWD		(1<<10)		// power-down enabled
#define Flash_CR_LVDENA		(0<<9)		// low voltage detector enabled
#define Flash_CR_LVDDIS		(1<<9)		// low voltage detector disabled
#define Flash_CR_BUSCFG		(1<<4)		// bus clock speed > 66MHz

#endif //__STR91x_FMI_H__
