#ifndef __STR91x_SCU_H__
#define __STR91x_SCU_H__

typedef struct SCU_t {
    uint32_t	clkcntr;	//+00	clock control			- reset by power-on only
    uint32_t	pllconf;	//+04	PLL configuration		- reset by power-on only
    uint32_t	sysstatus;	//+08	system status			- reset by power-on only
    uint32_t	pwrmng;		//+0C	power management
    uint32_t	itcmask;	//+10	interrupt mask
    uint32_t	pcgr0;		//+14	peripheral clock gating
    uint32_t	pcgr1;		//+18
    uint32_t	prr0;		//+1C	peripheral reset
    uint32_t	prr1;		//+20
    uint32_t	mgr0;		//+24	idle mode gating
    uint32_t	mgr1;		//+28
    uint32_t	pecgr0;		//+2C	peripheral emulation clock gating
    uint32_t	pecgr1;		//+30
    uint32_t	scr0;		//+34	system configuration
    uint32_t	scr1;		//+38	timers 0+1 prescaler
    uint32_t	scr2;		//+3C	timers 2+3 prescaler
    uint32_t	_40;
    uint32_t	gpioout[8];	//+44	GPIO output control
    uint32_t	gpioin[8];	//+64	GPIO input  control
    uint32_t	gpiotype[10];	//+84	GPIO output type
    uint32_t	emi;		//+AC	external memory interface
    uint32_t	wkupsel;	//+B0	wakeup selection
    uint32_t	_B4[2];
    uint32_t	gpioana;	//+BC	GPIO analog mode select
} SCU_t;

#define SCU_CLK_EMI_FULL	(0<<17)		// EMI bus clock = HCLK/1
#define SCU_CLK_EMI_HALF	(1<<17)		// EMI bus clock = HCLK/2
#define SCU_CLK_FMI_FULL	(0<<16)		// FMI clock = RCLK/1
#define SCU_CLK_FMI_HALF	(1<<16)		// FMI clock = RCLK/2
#define SCU_CLK_TIM23_EXT	(1<<14)		// Timers 2+3 clocked externally
#define SCU_CLK_TIM01_EXT	(1<<13)		// Timers 0+1 clocked externally
#define SCU_CLK_PHY		(1<<12)		// enable PHY clock output
#define SCU_CLK_USB_FULL	(0<<10)		// USB clock = MSTR/1
#define SCU_CLK_USB_HALF	(1<<10)		// USB clock = MSTR/2
#define SCU_CLK_USB_EXT		(2<<10)		// USB clock = external
#define SCU_CLK_BR_HALF		(0<<9)		// Baud rate clock = MSTR/2
#define SCU_CLK_BR_FULL		(1<<9)		// Baud rate clock = MSTR/1
#define SCU_CLK_PCLK_DIV1	(0<<7)		// PCLK / APB clock = RCLK/1
#define SCU_CLK_PCLK_DIV2	(1<<7)		// PCLK / APB clock = RCLK/2
#define SCU_CLK_PCLK_DIV4	(2<<7)		// PCLK / APB clock = RCLK/4
#define SCU_CLK_PCLK_DIV8	(3<<7)		// PCLK / APB clock = RCLK/8
#define SCU_CLK_HCLK_DIV1	(0<<5)		// HCLK / AHB clock = RCLK/1
#define SCU_CLK_HCLK_DIV2	(2<<5)		// HCLK / AHB clock = RCLK/2
#define SCU_CLK_HCLK_DIV4	(4<<5)		// HCLK / AHB clock = RCLK/4
#define SCU_CLK_RCLK_DIV1	(0<<2)		// RCLK = MSTR/1
#define SCU_CLK_RCLK_DIV2	(1<<2)		// RCLK = MSTR/2
#define SCU_CLK_RCLK_DIV4	(2<<2)		// RCLK = MSTR/4
#define SCU_CLK_RCLK_DIV8	(3<<2)		// RCLK = MSTR/8
#define SCU_CLK_RCLK_DIV16	(4<<2)		// RCLK = MSTR/16
#define SCU_CLK_RCLK_DIV1024	(5<<2)		// RCLK = MSTR/1024
#define SCU_CLK_MCLK_PLL	(0<<0)		// MCLK / MSTR = PLL
#define SCU_CLK_MCLK_RTC	(1<<0)		// MCLK / MSTR = RTC
#define SCU_CLK_MCLK_OSC	(2<<0)		// MCLK / MSTR = Oscillator
#define SCU_CLK_MCLK_MASK	(3<<0)

#define SCU_PLL_EN		(1<<19)		// enable PLL
#define SCU_PLL_PDIV(n)		((n)<<16)	// post-divider		(default: 3	Errata 1.9)
#define SCU_PLL_NDIV(n)		((n)<<8)	// feedback divider	(default: 192	Errata 1.9)
#define SCU_PLL_MDIV(n)		((n)<<0)	// pre-divider		(default: 25	Errata 1.9)
#if F_CRYSTAL == 25000000
  #define SCU_PLL_96MHz		(SCU_PLL_MDIV(25)|SCU_PLL_NDIV(192)|SCU_PLL_PDIV(2))
  #define SCU_PLL_48MHz		(SCU_PLL_MDIV(25)|SCU_PLL_NDIV(192)|SCU_PLL_PDIV(3))
#endif

// write 1 to reset event:
#define SCU_SYS_SRAM_ERR	(1<<5)		// SRAM error event - write to invalid location
#define SCU_SYS_PFQBC_ACK	(1<<4)		// PFQBC acknowledge event - see SCU_SCR0
#define SCU_SYS_LVD_RST		(1<<3)		// device was reset by low voltage detector
#define SCU_SYS_WDG_RST		(1<<2)		// device was reset by watchdog
#define SCU_SYS_LOCK_LOST	(1<<1)		// PLL lost lock
#define SCU_SYS_LOCK		(1<<0)		// PLL locked

#define SCU_PWR_FLASH_DBG	(1<<4)		// power-down flash during debug mode
#define SCU_PWR_CPU_INTR	(1<<3)		// interrupt code executes at full speed
#define SCU_PWR_RUN		(0<<0)		// run mode
#define SCU_PWR_IDLE		(1<<0)		// idle mode
#define SCU_PWR_SLEEP		(2<<0)		// sleep mode

#define SCU_ITC_LVD_RST_DIS	(1<<4)		// disable reset when low voltage detected
#define SCU_ITC_SRAM_ERR_DIS	(1<<3)		// disable interrupt on SRAM error (invalid address)
#define SCU_ITC_PFQB_ACK_DIS	(1<<2)		// disable interrupt on PFQBC acknowledge
#define SCU_ITC_LOCK_LOST_DIS	(1<<1)		// disable interrupt on lost PLL lock
#define SCU_ITC_LOCK_DIS	(1<<0)		// disable interrupt on PLL lock

// PCGR, PRR,...				//		PCGR	PRR	MGR	PECGR
#define SCU_PCGR0_MASK		0x00000FFB
#define SCU_PRR0_MASK		0x00001D73
#define SCU_MGR0_MASK		0x00000FFA
#define SCU_MECGR0_MASK		0x00000FFB
#define SCU_PR0_PFQBC_AHB	(1<<12)		//		-	x	-	-
#define SCU_PR0_MAC		(1<<11)		//		x	x	x	x
#define SCU_PR0_USB48M		(1<<10)		// USB 48MHz	x	-	x	x
#define SCU_PR0_USB		(1<<9)		//		x	x	x	x
#define SCU_PR0_DMA		(1<<8)		//		x	x	x	x
#define SCU_PR0_EXT_MEM		(1<<7)		// ext. memory	x	-	x	x
#define SCU_PR0_EMI		(1<<6)		//		x	x	x	x
#define SCU_PR0_VIC		(1<<5)		//		x	x	x	x
#define SCU_PR0_SRAM_ARB	(1<<4)		// SRAM arbiter	x	x	x	x
#define SCU_PR0_SRAM		(1<<3)		//		x	-	x	x
#define SCU_PR0_PFQBC		(1<<1)		//		x	x	x	x
#define SCU_PR0_FMI		(1<<0)		//		x	x	-	x

#define SCU_PCGR1_MASK		0x01FFFFFF
#define SCU_PRR1_MASK		0x01FFFFFF
#define SCU_MGR1_MASK		0x01FFFFFF
#define SCU_MECGR1_MASK		0x01FFFFFF
#define SCU_PR1_RTC		(1<<24)		//		x	x	x	x
#define SCU_PR1_GPIO(n)		(1<<((n)+14))	//		x	x	x	x
#define SCU_PR1_WIU		(1<<13)		//		x	x	x	x
#define SCU_PR1_WDG		(1<<12)		//		x	x	x	x
#define SCU_PR1_ADC		(1<<11)		//		x	x	x	x
#define SCU_PR1_CAN		(1<<10)		//		x	x	x	x
#define SCU_PR1_SSP(n)		(1<<((n)+8))	//		x	x	x	x
#define SCU_PR1_I2C(n)		(1<<((n)+6))	//		x	x	x	x
#define SCU_PR1_UART(n)		(1<<((n)+3))	//		x	x	x	x
#define SCU_PR1_MC		(1<<2)		//		x	x	x	x
#define SCU_PR1_TIM23		(1<<1)		//		x	x	x	x
#define SCU_PR1_TIM01		(1<<0)		//		x	x	x	x

#define SCU_SCR0_P30_EMT	(1<<14)		// P3.0 configured for external debug
#define SCU_SCR0_P30_TRG	(1<<13)		// P3.0 debug function select
#define SCU_SCR0_IRDA(n)	(1<<((n)+10))	// Enable IrDA enabled for UART
#define SCU_SCR0_EMI_ALE_LNGT	(1<<8)		// EMI ALE length
#define SCU_SCR0_EMI_ALE_POLR	(1<<7)		// EMI ALE polarity
#define SCU_SCR0_EMI_MUX	(1<<6)		// EMI multiplex select
#define SCU_SCR0_SRAM_LK_EN	(1<<5)		// Enable SRAM arbiter lock tramsferr
#define SCU_SCR0_SRAM_32KB	(0<<3)		// SRAM size 32KB
#define SCU_SCR0_SRAM_64KB	(1<<3)		// SRAM size 64KB
#define SCU_SCR0_SRAM_96KB	(2<<3)		// SRAM size 96KB
#define SCU_SCR0_AHB_0WS	(0<<2)		// AHB 0 wait states
#define SCU_SCR0_AHB_1WS	(1<<2)		// AHB 1 wait state
#define SCU_SCR0_DTCM_0WS	(0<<1)		// DTCM 0 wait states
#define SCU_SCR0_DTCM_1WS	(1<<1)		// DTCM 1 wait state
#define SCU_SCR0_PFQBC_EN	(1<<0)		// Enable PFQBC

#define SCU_GOUT_IN(b)		(0<<((b)*2))	// bit <n> is GPIO/peripheral input
#define SCU_GOUT_OUT(b)		(1<<((b)*2))	// bit <n> is GPIO output
#define SCU_GOUT_ALT2(b)	(2<<((b)*2))	// bit <n> is peripheral output 2
#define SCU_GOUT_ALT3(b)	(3<<((b)*2))	// bit <n> is peripheral output 3

#define SCU_GIN_IN(b)		(0<<(b))	// bit <n> is GPIO/peripheral input
#define SCU_GIN_ALT1(b)		(1<<(b))	// bit <n> is peripheral input 2

#define SCU_GTYPE_OC(b)		(1<<(b))	// bit <n> is open collector

#define SCU_EMI_GPIO		(0<<0)		// Port 8+9 is GPIO
#define SCU_EMI_ENABLE		(1<<0)		// Port 8+9 is EMI

// Wakup by / connect to interrupt source VIC1.10 (VIC_WIU_GRP0)
#define SCU_WKUP0_RTC		(0<<0)
#define SCU_WKUP0_USB		(1<<0)
#define SCU_WKUP0_EXTINT2	(2<<0)
#define SCU_WKUP0_EXTINT3	(3<<0)
#define SCU_WKUP0_EXTINT4	(4<<0)
#define SCU_WKUP0_EXTINT5	(5<<0)
#define SCU_WKUP0_EXTINT6	(6<<0)
#define SCU_WKUP0_EXTINT7	(7<<0)
// Wakup by / connect to interrupt source VIC1.11 (VIC_WIU_GRP1)
#define SCU_WKUP1_EXTINT8	(0<<3)
#define SCU_WKUP1_EXTINT9	(1<<3)
#define SCU_WKUP1_EXTINT10	(2<<3)
#define SCU_WKUP1_EXTINT11	(3<<3)
#define SCU_WKUP1_EXTINT12	(4<<3)
#define SCU_WKUP1_EXTINT13	(5<<3)
#define SCU_WKUP1_EXTINT14	(6<<3)
#define SCU_WKUP1_EXTINT15	(7<<3)
// Wakup by / connect to interrupt source VIC1.12 (VIC_WIU_GRP2)
#define SCU_WKUP2_EXTINT16	(0<<6)
#define SCU_WKUP2_EXTINT17	(1<<6)
#define SCU_WKUP2_EXTINT18	(2<<6)
#define SCU_WKUP2_EXTINT19	(3<<6)
#define SCU_WKUP2_EXTINT20	(4<<6)
#define SCU_WKUP2_EXTINT21	(5<<6)
#define SCU_WKUP2_EXTINT22	(6<<6)
#define SCU_WKUP2_EXTINT23	(7<<6)
// Wakup by / connect to interrupt source VIC1.13 (VIC_WIU_GRP3)
#define SCU_WKUP3_EXTINT24	(0<<9)
#define SCU_WKUP3_EXTINT25	(1<<9)
#define SCU_WKUP3_EXTINT26	(2<<9)
#define SCU_WKUP3_EXTINT27	(3<<9)
#define SCU_WKUP3_EXTINT28	(4<<9)
#define SCU_WKUP3_EXTINT29	(5<<9)
#define SCU_WKUP3_EXTINT30	(6<<9)
#define SCU_WKUP3_EXTINT31	(7<<9)

#define SCU_ANA_ENABLE(b)	(1<<(b))	// Enable analog input on pin P4.<b>

#endif //__STR91x_SCU_H__
