#ifndef __STR91x_TIM_H__
#define __STR91x_TIM_H__

typedef struct TIM_t {
    uint32_t	ic1r;		//+00 input capture register 1
    uint32_t	ic2r;		//+04 input capture register 1
    uint32_t	oc1r;		//+08 output compare register 1
    uint32_t	oc2r;		//+0C output compare register 1
    uint32_t	cntr;		//+10 counter register
    uint32_t	cr1;		//+14 control register 1
    uint32_t	cr2;		//+14 control register 2
    uint32_t	sr;		//+18 status register
} TIM_t;

#define TIM_CR1_EN		(1<<15)		// counter enabled
#define TIM_CR1_PWMI		(1<<14)		// PWM input mode enabled
#define TIM_CR1_IC1_DMA		(0<<12)		// DMA source is IC1
#define TIM_CR1_OC1_DMA		(1<<12)		// DMA source is OC1
#define TIM_CR1_IC2_DMA		(2<<12)		// DMA source is IC2
#define TIM_CR1_OC2_DMA		(3<<12)		// DMA source is OC2
#define TIM_CR1_FOLV2		(1<<11)		// force OLV2 on OCMP2 pin
#define TIM_CR1_FOLV1		(1<<10)		// force OLV1 on OCMP1 pin
#define TIM_CR1_OLV2		(1<<9)		// value copied to OCMP2 pin on match
#define TIM_CR1_OLV1		(1<<8)		// value copied to OCMP1 pin on match
#define TIM_CR1_OC2E		(1<<7)		// output compare 2 enable
#define TIM_CR1_OC1E		(1<<6)		// output compare 1 enable
#define TIM_CR1_OPM		(1<<5)		// one pulse mode
#define TIM_CR1_PWM		(1<<4)		// pulse width modulation mode
#define TIM_CR1_IEGD2		(1<<3)		// input capture 2 on 0:falling 1:rising edge
#define TIM_CR1_IEGD1		(1<<2)		// input capture 1 on 0:falling 1:rising edge
#define TIM_CR1_EXEDG		(1<<1)		// count on external 0:falling 1:rising edge
#define TIM_CR1_EXKEN		(1<<0)		// external clock enable

#define TIM_CR2_IC1IE		(1<<15)		// input capture 1 interrupt enable
#define TIM_CR2_OC1IE		(1<<14)		// output capture 1 interrupt enable
#define TIM_CR2_TOIE		(1<<13)		// timer overflow interrupt enable
#define TIM_CR2_IC2IE		(1<<12)		// input capture 1 interrupt enable
#define TIM_CR2_OC2IE		(1<<11)		// output capture 2 interrupt enable
#define TIM_CR2_DMAE		(1<<10)		// DMA enable
#define TIM_CR2_CC(n)		(((n)-1)&0xFF)	// clock prescaler, PCLK/1..256

#define TIM_SR_ICF1		(1<<15)		// input capture flag 1
#define TIM_SR_OCF1		(1<<14)		// output compare flag 1
#define TIM_SR_TOF		(1<<13)		// timer overflow flag
#define TIM_SR_ICF2		(1<<12)		// input capture flag 2
#define TIM_SR_OCF2		(1<<11)		// output compare flag 2

#endif //__STR91x_TIM_H__
